//_CRT_SECURE_NO_DEPRECATE

//OpenCV Includes
#ifdef _CH_
#pragma package <opencv>
#endif

#ifndef _EiC
#include "cv.h"
#include "highgui.h"
#include <stdio.h>
#include <ctype.h>
#endif

#include <cvcam.h>
#include <sstream>
#include <algorithm>

//My includes
#include <iostream>
#include <fstream>
#include <string>
#include <math.h>
#include <time.h>

struct dotData
{
	double ix;
	double iy;
	
	double area;

	double wx;
	double wy;
	double wz;
};


struct target
{
	int numDots;		//Number of Dots
	dotData dots[9];	//Individual Dot Information
	dotData group;		//Joint information
	dotData minDot;

	double angle;
	double angleoffset;
	double zMoment;
	
	int x1, y1, x2, y2;

	int num_idd;
	double idd[36];
	double smallest_idd;
	//CvMoments my_mom;
	//CvHuMoments hu_mom;

	target *nxt;
};

struct classifier
{
	int ID;
	int numDots;
	double angleoffset;

	int num_idd;
	double idd[36];
	double smallest_idd;

	classifier *nxt;
};



using namespace std;


double compare (const void * a, const void * b)
{
	return (double) ( *(double*)a - *(double*)b);
}


void q_sort(double numbers[], int left, int right)
{
   double pivot, l_hold, r_hold;
   l_hold = left;
   r_hold = right;
   pivot = numbers[left];
   while (left < right)
   {
      while ((numbers[right] >= pivot) && (left < right))
      right--;
      if (left != right)
      {
	 numbers[left] = numbers[right];
	 left++;
      }
      while ((numbers[left] <= pivot) && (left < right))
	 left++;
      if (left != right)
      {
	  numbers[right] = numbers[left];
	  right--;
      }
   }
   numbers[left] = pivot;
   pivot = left;
   left = l_hold;
   right = r_hold;
   if (left < pivot)
      q_sort(numbers, left, pivot-1);
   if (right > pivot)
      q_sort(numbers, pivot+1, right);
}

void on_mouse( int event, int x, int y, int flags, void* param )
{
    
	if( event == CV_EVENT_LBUTTONDOWN )
	{
		cout<<" X:"<<x<<" Y: "<<y<<endl;
	}

	if( event == CV_EVENT_RBUTTONDOWN )
	{
		cout<<"x: "<<x<<" y: "<<y<<endl;
	}
}

classifier* LoadTargetData()
{
	ifstream fin("target_classifiers.txt");
	string line;

	classifier* current = new classifier;
	classifier* first = current;
	classifier* previous = new classifier;
	while(!fin.eof())
	{
		//create a new classifier and initialize it
		classifier* new_classifier = new classifier;
		new_classifier->nxt = NULL;
		new_classifier->ID = -1;
		new_classifier->numDots = -1;
		new_classifier->angleoffset = -1;

		//read one line of data from txt file
		fin>>line;

		string data;
		int idd_num = 0;
		//process data, looking for commas
		while(line.find(",")!=string::npos)
		{
			data = line.substr(0, line.find(","));
			line = line.substr(line.find(",") + 1);
			
			if (new_classifier->ID == -1)
			{
				new_classifier->ID = atof(data.c_str());
			}
			else if (new_classifier->numDots == -1)
			{
				new_classifier->numDots= atof(data.c_str());
			}
			else if (new_classifier->angleoffset == -1)
			{
				new_classifier->angleoffset = atof(data.c_str());
			}
			else
			{
				new_classifier->idd[idd_num] = atof(data.c_str());
				idd_num++;
			}
		}
		current->num_idd = idd_num;
		previous = current;
		current->nxt = new_classifier;
		current = current->nxt;
	}
	previous->nxt = NULL;
	first = first->nxt;
	return first;
	fin.close();
	

}

int main()
{
	classifier* first_classifier;
	first_classifier = LoadTargetData();

	classifier* current;
	current = first_classifier;
	while(current)
	{
		cout<<"ID: "<<current->ID<<" num_dots: "<<current->numDots<<" IDD[0]: "<<current->idd[0]<<endl;
		current = current->nxt;
	}


	cvNamedWindow("Training 1", 0);
	cvSetMouseCallback( "Training 1", on_mouse, 0 );

	ofstream fout("target_classifiers.txt");
	
	IplImage* img=0; 
	IplImage* grey = 0;

	for (int target_id = 1; target_id<25; target_id++)
	{

		string fname;
		fname = "images\\FinalSet\\";
		ostringstream oss;
		oss<<target_id;
		string s;
		s = oss.str();
		fname.append(s);
		fname.append(".bmp");
		
		
		img=cvLoadImage(fname.c_str());
		grey = cvCreateImage(cvGetSize(img), 8, 1);
		cvCvtColor(img, grey, CV_BGR2GRAY );
		cvThreshold(grey, grey, 100, 255, CV_THRESH_BINARY_INV);

		if(!img)
		{
			cout<<"Could not load image.\n";
		}
		cvShowImage("Training 1", grey);
		
		//find num Dots
		target *my_target = new target;
		CvMemStorage* storage = cvCreateMemStorage(0);
		CvSeq* contour = 0;
		my_target->numDots = cvFindContours(grey, storage, &contour, sizeof(CvContour), CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE );
				
		//Find interdottal distances
		int i = 0;
		for(contour;contour; contour=contour->h_next)
		{
			CvMoments moments;
			cvContourMoments(contour, &moments);
			double area = cvGetSpatialMoment(&moments, 0, 0);
			double centerx = cvGetSpatialMoment(&moments, 1, 0) / area;
			double centery = cvGetSpatialMoment(&moments, 0, 1) / area;
			my_target->dots[i].wx = centerx;
			my_target->dots[i].wy = centery;
			cvCircle(img, cvPoint(centerx, centery), 10, CV_RGB(255, 0, 0), -1);
			i++;
		}

		//find group center of mass
		{
			my_target->group.wx = 0;
			my_target->group.wy = 0;

			for(int i = 0; i<my_target->numDots; i++)
			{
				my_target->group.wx += my_target->dots[i].wx;
				my_target->group.wy += my_target->dots[i].wy;
			}
			my_target->group.wx /= my_target->numDots;
			my_target->group.wy /= my_target->numDots;
		}


		//find closest point to center of mass and angle
		{
			double min_dist = -1;
			for(int i = 0; i<my_target->numDots; i++)
			{
				double temp_dist = sqrt(pow((my_target->group.wx - my_target->dots[i].wx),2) + pow( (my_target->group.wy  - my_target->dots[i].wy),2));
				if (temp_dist < min_dist || min_dist == -1)
				{
					min_dist = temp_dist;
					my_target->minDot = my_target->dots[i];
				}
			}
			cvCircle(img, cvPoint(my_target->group.wx, my_target->group.wy), 10, CV_RGB(0, 255, 0), -1);
			cvCircle(img, cvPoint(my_target->minDot.wx, my_target->minDot.wy), 10, CV_RGB(0, 0, 255), -1);
		}


		//Find angle offset
		{
			if (my_target->minDot.wx < my_target->group.wx)
			{
				if(my_target->minDot.wy < my_target->group.wy)		//quadrant 2
				{
					cvCircle(img, cvPoint(my_target->minDot.wx, my_target->minDot.wy), 10, CV_RGB(0, 0, 255), -1);
					my_target->angleoffset = atan((my_target->minDot.wy - my_target->group.wy)/(my_target->minDot.wx - my_target->group.wx));
					my_target->angleoffset = 180 - my_target->angleoffset*180.0/3.14159;
					cout<<"("<<my_target->minDot.wx<<","<<my_target->minDot.wy<<") ";
					cout<<"("<<my_target->group.wx<<","<<my_target->group.wy<<") ";
					cout<<"angle: "<<my_target->angleoffset<<endl;
				}
				else												//quadrant 3
				{
					cvCircle(img, cvPoint(my_target->minDot.wx, my_target->minDot.wy), 10, CV_RGB(0, 0, 128), -1);
					my_target->angleoffset = atan((my_target->minDot.wy - my_target->group.wy)/(my_target->minDot.wx - my_target->group.wx));
					my_target->angleoffset = 180 - my_target->angleoffset*180.0/3.14159;
					cout<<"("<<my_target->minDot.wx<<","<<my_target->minDot.wy<<") ";
					cout<<"("<<my_target->group.wx<<","<<my_target->group.wy<<") ";
					cout<<"angle: "<<my_target->angleoffset<<endl;
				}
			}
			else
			{
				if(my_target->minDot.wy < my_target->group.wy)		//quadrant 1
				{
					cvCircle(img, cvPoint(my_target->minDot.wx, my_target->minDot.wy), 10, CV_RGB(128, 128, 0), -1);
					my_target->angleoffset = atan((my_target->minDot.wy - my_target->group.wy)/(my_target->minDot.wx - my_target->group.wx));
					my_target->angleoffset = -my_target->angleoffset*180.0/3.14159;
					cout<<"("<<my_target->minDot.wx<<","<<my_target->minDot.wy<<") ";
					cout<<"("<<my_target->group.wx<<","<<my_target->group.wy<<") ";
					cout<<"angle: "<<my_target->angleoffset<<endl;
				}
				else												//quandrant 4
				{
					cvCircle(img, cvPoint(my_target->minDot.wx, my_target->minDot.wy), 10, CV_RGB(255, 255, 0), -1);
					my_target->angleoffset = atan((my_target->minDot.wy - my_target->group.wy)/(my_target->minDot.wx - my_target->group.wx));
					my_target->angleoffset = -my_target->angleoffset*180.0/3.14159;
					cout<<"("<<my_target->minDot.wx<<","<<my_target->minDot.wy<<") ";
					cout<<"("<<my_target->group.wx<<","<<my_target->group.wy<<") ";
					cout<<"angle: "<<my_target->angleoffset<<endl;
				}
			}
		}


		cvShowImage("Training 1", img);
		cvWaitKey(0);

		int count = 0;
		for(int i = 0; i<my_target->numDots; i++)
		{
			for(int j = i+1; j<my_target->numDots; j++)
			{
				double dist = sqrt(pow((my_target->dots[i].wx - my_target->dots[j].wx),2) + pow( (my_target->dots[i].wy - my_target->dots[j].wy),2));
				my_target->idd[count] = dist;
				count++;
			}
		}
		my_target->num_idd = count;
		my_target->smallest_idd = my_target->idd[0];
		for(int i = 0; i<my_target->num_idd; i++)
		{
			if (my_target->idd[i]<my_target->smallest_idd)
			{	
				my_target->smallest_idd = my_target->idd[i];
			}
		}


		//Normalize to smallest idd
		for(int i = 0; i<my_target->num_idd; i++)
		{
			my_target->idd[i] /= my_target->smallest_idd;
		}
		
		//sort the distances smallest to largest
		q_sort(my_target->idd, 0, count-1);
		

		//write output
		fout<<target_id<<","<<my_target->numDots<<","<<my_target->angleoffset;
		for(int i = 0; i<my_target->num_idd; i++)
		{
			fout<<","<<my_target->idd[i];
		}
		fout<<endl;

	}

	cvWaitKey(0);
	cvDestroyWindow("Training 1");
	cvReleaseImage(&img);
	cvReleaseImage(&grey);
    return 0;
}

#ifdef _EiC
main(1,"lkdemo.c");
#endif
